package JPAARealTimePackage 
  "Package to attach real time capabilities to models" 
  
protected 
function init "Initializes the hardware timer on mainboard" 
    
    annotation (Include="#include <TimerAccess.h>", Library="TimerAccessLibrary");
    
    external "C" init();
    
  annotation (preferedView="info",Documentation(info="<HTML>
<h3><font color=\"#008000\">Description</font></h3>
<p>
Function <b>init</b>() resets the hardware timer value and starts the timer. To get the value use <b>getTime</b>().
</p>
</HTML>"));
end init;
  
function sleep "returns the current hardware timer value of mainboard" 
    
    annotation (Include="#include <TimerAccess.h>", Library="TimerAccessLibrary");
    
    input Integer sleepTime "time to pause the simulation in ms";
    external "C" sleep(sleepTime);
    
  annotation (preferedView="info",Documentation(info="<HTML>
<h3><font color=\"#008000\">Description</font></h3>
<p>
Function <b>sleep</b>(Integer sleepTime) pauses the simulation for sleepTime ms.
</p>
</HTML>"));
end sleep;
  
  function getTime "returns the current hardware timer value of mainboard" 
    
    annotation (Include="#include <TimerAccess.h>", Library="TimerAccessLibrary");
    output Modelica.SIunits.Time currentTime;
    
    external "C" currentTime=  getTime();
    
  annotation (preferedView="info",Documentation(info="<HTML>
<h3><font color=\"#008000\">Description</font></h3>
<p>
Function <b>getTime</b>() returns the time elapsed since start or reset of timer in seconds.
</p>
</HTML>"));
  end getTime;
  
  annotation (uses(Modelica(version="2.2.1")));
  
public 
  model JPAARealTime 
    "Model that forces the simulation to run in predefined speed" 
    
    parameter Real averageSimulationSpeed=1 
      "simulation time divided by real time";
    parameter Modelica.SIunits.Time sleepTime=0.001 
      "Time to send the simulation to sleep if it runs to fast";
    
    parameter Real maxSimulationSpeed=1.1 "Maximum simulation speed";
    parameter Modelica.SIunits.Time samplingTime=0.5 
      "Samplingtime for simulation speed check";
    
    output Modelica.SIunits.Time realTime;
    output Real simulationSpeed;
    output Modelica.SIunits.Time timeDifference;
    output Modelica.SIunits.Time simulationTime;
    
  protected 
    Modelica.SIunits.Time oldRealTime;
    Modelica.SIunits.Time oldSimulationTime;
    Boolean s;
    
  algorithm 
    //on simulation startup reset and start mainboard timer
    if (initial()) then
      init();
    end if;
    
    //create event to update simulation speed reference point in time
    s:=sample(0, samplingTime);
    
    //on event update simulation speed reference point
    when 
        (s) then
      oldSimulationTime:=time;
      oldRealTime:=getTime();
    end when;
    
    //update time based variables
    realTime:=getTime();
    simulationTime:=time;
    timeDifference:=getTime() - time;
    
    //calculate simulation speed as quotient of elapsed simulation time and elapsed real time
    simulationSpeed:=(time - oldSimulationTime)/(realTime - oldRealTime);
    
    //as long as simulation time is ahead real time or simulationspeed exceeds maximum simulation speed pause simulation
    while 
         ((time > realTime * averageSimulationSpeed or simulationSpeed>maxSimulationSpeed)) loop
      realTime:=getTime();
      simulationSpeed:=(time - oldSimulationTime)/(realTime - oldRealTime);
      
      if (sleepTime > 0) then
        sleep(integer(ceil((sleepTime*1000))));
      end if;
    end while;
    
    annotation (Icon(
        Ellipse(extent=[-90,80; -10,0], style(color=3, rgbcolor={0,0,255})),
        Polygon(points=[-50,80; -46,40; -54,40; -50,80], style(
            color=3,
            rgbcolor={0,0,255},
            fillColor=0,
            rgbfillColor={0,0,0})),
        Polygon(points=[-50,46; -46,40; -86,24; -50,46], style(
            color=3,
            rgbcolor={0,0,255},
            fillColor=0,
            rgbfillColor={0,0,0})),
        Ellipse(extent=[12,0; 92,-80], style(color=3, rgbcolor={0,0,255})),
        Polygon(points=[52,0; 56,-40; 48,-40; 52,0], style(
            color=3,
            rgbcolor={0,0,255},
            fillColor=0,
            rgbfillColor={0,0,0})),
        Polygon(points=[52,-34; 56,-40; 16,-56; 52,-34], style(
            color=3,
            rgbcolor={0,0,255},
            fillColor=0,
            rgbfillColor={0,0,0})),
        Line(points=[-80,-80; 80,80], style(
            color=3,
            rgbcolor={0,0,255},
            pattern=4,
            thickness=4,
            fillColor=0,
            rgbfillColor={0,0,0},
            fillPattern=1)),
        Text(
          extent=[-100,100; 60,80],
          style(
            color=3,
            rgbcolor={0,0,255},
            pattern=4,
            thickness=4,
            fillColor=0,
            rgbfillColor={0,0,0},
            fillPattern=1),
          string="Simulation"),
        Text(
          extent=[-60,-80; 100,-100],
          style(
            color=3,
            rgbcolor={0,0,255},
            pattern=4,
            thickness=4,
            fillColor=0,
            rgbfillColor={0,0,0},
            fillPattern=1),
          string="Real World"),
        Line(points=[-86,24; 16,-56], style(
            color=10,
            rgbcolor={135,135,135},
            thickness=2,
            fillColor=0,
            rgbfillColor={0,0,0},
            fillPattern=1)),
        Line(points=[-50,80; 52,0], style(
            color=10,
            rgbcolor={135,135,135},
            thickness=2,
            fillColor=0,
            rgbfillColor={0,0,0},
            fillPattern=1)),
        Ellipse(extent=[-52,80; -48,78], style(
            color=10,
            rgbcolor={135,135,135},
            thickness=2,
            fillColor=0,
            rgbfillColor={0,0,0},
            fillPattern=1)),
        Ellipse(extent=[50,0; 54,-2], style(
            color=10,
            rgbcolor={135,135,135},
            thickness=2,
            fillColor=0,
            rgbfillColor={0,0,0},
            fillPattern=1)),
        Ellipse(extent=[14,-54; 18,-56], style(
            color=10,
            rgbcolor={135,135,135},
            thickness=2,
            fillColor=0,
            rgbfillColor={0,0,0},
            fillPattern=1)),
        Ellipse(extent=[-88,26; -84,24], style(
            color=10,
            rgbcolor={135,135,135},
            thickness=2,
            fillColor=0,
            rgbfillColor={0,0,0},
            fillPattern=1))));
  annotation (
      Documentation(info="<html>
<p>This model forces the simulation to run with predefined speed.</p>
<h4>Parametrization</h4>
<p>The model has four parameters which specify in which way the simulation should be slowed down.</p>
<p>
  <ul>
    <li>averageSimulationSpeed: Specifies the desired simulation speed. For real time simulations this parameter needs to be set to 1.
If the parameter is larger than 1 the simulation runs faster than real time If it is less than 1 then it runs slower than real time.</li>
    <li>sleepTime: Specifies how many seconds the Simulation should be sent to sleep if simulation is to fast. If the value equals 0 the simulation is not sent to sleep. Normally you should sent the simulation asleep at least for a millisecond. Otherwise simulation consumes all the processor ressources and the computer will not react on user inputs for a while. Values smaller than 0.001 are identified as 0.</li>  
    <li>maximumSimulationSpeed: Specifies the maximum simulation speed which is allow to regain simulation previously lost. This value needs to be somewhat larger than averageSimulationSpeed. Otherwise averageSimulationSpeed cannot be reached due to synchronizing overhead.</li>
    <li>samplingTime: Specifies the sampling interval to reset point in time for calculation of simulation Speed.</li>
    <li>
  </ul>
</p>
</html>
"));
  end JPAARealTime;
  
  model Example "Example that shows how to attach JPAARealTime to a model" 
    
    Modelica.Blocks.Continuous.SecondOrder secondOrder(w=10) 
      annotation (extent=[30,10; 50,30]);
    Modelica.Blocks.Continuous.PI PI(T=0.1) annotation (extent=[-10,10; 10,30]);
    annotation (Diagram,
      experiment(StopTime=10),
      experimentSetupOutput);
    Modelica.Blocks.Math.Feedback feedback annotation (extent=[-50,10; -30,30]);
    Modelica.Blocks.Sources.Step step(
      height=1,
      offset=0,
      startTime=0.1) annotation (extent=[-92,10; -72,30]);
    JPAARealTime jPAARealTime annotation (extent=[-90,70; -70,90]);
  equation 
    connect(PI.y, secondOrder.u) 
      annotation (points=[11,20; 28,20], style(color=74, rgbcolor={0,0,127}));
    connect(feedback.y, PI.u) annotation (points=[-31,20; -12,20], style(color=
            74, rgbcolor={0,0,127}));
    connect(secondOrder.y, feedback.u2) annotation (points=[51,20; 70,20; 70,
          -20; -40,-20; -40,12], style(color=74, rgbcolor={0,0,127}));
    connect(step.y, feedback.u1) annotation (points=[-71,20; -48,20], style(
          color=74, rgbcolor={0,0,127}));
  end Example;
annotation (
      Documentation(info="<html>
<p>JPAARealTimePackage enables Modelica models to run in real time in Dymola IDE. The Dymola Real Time option is not needed.</p>
<h4>Author</h4>
<p>
  Florian Wagner<br>
  TU Kaiserslautern<br>
  FB EIT, JPA<br>
  Erwin-Schrdinger-Str. 12<br>
  67663 Kaiserslautern<br>
  Germany<br>
  eMail: <a href=\"mailto:wagner@eit.uni-kl.de\">wagner@eit.uni-kl.de</a><br>
  WWW: <a href=\"http://www.eit.uni-kl.de/frey\">http://www.eit.uni-kl.de/frey</a>
</p>
<h4>Copyright holder</h4>
<p>
  TU Kaiserslautern<br>
  FB EIT, JPA<br>
  Erwin-Schrdinger-Str. 12<br>
  67663 Kaiserslautern<br>
  Germany<br>
</p>
<h4>License and Copyright</h4>
<p>Copyright (c) 2007 TU Kaiserslautern, FB EIT, JPA</p>
<p>Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the \"Software\"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:</p>
<p>The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.</p>
<p>THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.</p>
 
<h4>Differences to the Dymola Real Time Option</h4>
<p>The behaviour of the JPAARealTime package differs in three main aspects from the Dymola Real Time option</p>
<p>
  <ul>
    <li>Attachment to the simulated model: The Dymola Real Time option is assigned to a simulated model by the simulation settings. JPAARealTime is a component which is part of the simulated model.</li>
    <li>Simulation speed: The Dymola Real Time option limits the simulation progress, simulation time must never be ahead real time. But simulation speed is not limited. As long as simulation time is behind real time simulation runs as fast as possible. JPAARealTime additionally limits simulation speed. Previously gained deficit in simulation time is reduce with limited speed only (specified by the user).</li>
    <li>DDE: The Dymola Real Time option makes use of DDE to get information about real time. JPAARealTime makes direct calls to the Windows operating system instead.</li>
  </ul>
</p>
<h4>Requirements</h4>
<p>The JPAARealTime package has been developed for the Modelica IDE Dymola by Dynasim running on a Windows XP operating system. To use the package one needs the Microsoft Visual Studio 2005 C++ compiler. The package has been tested using Dymola 6.1 on Windows XP SP2 using Microsoft Visual Studio 2005 Professional Edition compiler.</p>
<h4>Installation</h4>
<p>To use the JPAARealTime package copy the files TimerAccess.h,  TimerAccessLibrary.lib and JPAARealTime.mo in the same directory as the model which makes use of JPAARealTime. To attach real time capabilities to a model simply instantiate JPAARealTime in it.</p>
<h4>Known Errors</h4>
<p>
  <ul>
    <li>JPAARealTime cannot be used together with DDE</li>
  </ul>
</p>
</html>
"));
end JPAARealTimePackage;
